@@ -1,24 +1,27 @@
+#include <cpp11.hpp>
 #include "pch.h"
 #include "SqliteResultImpl.h"
 #include "SqliteDataFrame.h"
 #include "DbColumnStorage.h"
+#include "DbConnection.h"
 #include "integer64.h"
 
 
 
 // Construction ////////////////////////////////////////////////////////////////
 
-SqliteResultImpl::SqliteResultImpl(sqlite3* conn_, const std::string& sql) :
-  conn(conn_),
-  stmt(prepare(conn_, sql)),
+SqliteResultImpl::SqliteResultImpl(const DbConnectionPtr& conn_, const std::string& sql) :
+  conn(conn_->conn()),
+  stmt(prepare(conn, sql)),
   cache(stmt),
   complete_(false),
   ready_(false),
   nrows_(0),
-  total_changes_start_(sqlite3_total_changes(conn_)),
+  total_changes_start_(sqlite3_total_changes(conn)),
   group_(0),
   groups_(0),
-  types_(get_initial_field_types(cache.ncols_))
+  types_(get_initial_field_types(cache.ncols_)),
+  with_alt_types_(conn_->with_alt_types())
 {
 
   LOG_DEBUG << sql;
@@ -76,14 +79,22 @@ std::vector<DATA_TYPE> SqliteResultImpl::get_initial_field_types(const size_t nc
 sqlite3_stmt* SqliteResultImpl::prepare(sqlite3* conn, const std::string& sql) {
   sqlite3_stmt* stmt = NULL;
 
+  const char* tail = NULL;
+
   int rc =
     sqlite3_prepare_v2(
       conn, sql.c_str(), (int)std::min(sql.size() + 1, (size_t)INT_MAX),
-      &stmt, NULL
+      &stmt, &tail
     );
   if (rc != SQLITE_OK) {
     raise_sqlite_exception(conn);
   }
+  if (tail) {
+    while (isspace(*tail)) ++tail;
+    if (*tail) {
+      cpp11::warning(std::string("Ignoring remaining part of query: ") + tail);
+    }
+  }
 
   return stmt;
 }
@@ -98,7 +109,10 @@ void SqliteResultImpl::init(bool params_have_rows) {
 
 // Publics /////////////////////////////////////////////////////////////////////
 
-bool SqliteResultImpl::complete() {
+void SqliteResultImpl::close() {
+}
+
+bool SqliteResultImpl::complete() const {
   return complete_;
 }
 
@@ -111,19 +125,19 @@ int SqliteResultImpl::n_rows_affected() {
   return sqlite3_total_changes(conn) - total_changes_start_;
 }
 
-void SqliteResultImpl::bind(const List& params) {
+void SqliteResultImpl::bind(const cpp11::list& params) {
   if (cache.nparams_ == 0) {
-    stop("Query does not require parameters.");
+    cpp11::stop("Query does not require parameters.");
   }
 
   if (params.size() != cache.nparams_) {
-    stop("Query requires %i params; %i supplied.",
+    cpp11::stop("Query requires %i params; %i supplied.",
          cache.nparams_, params.size());
   }
 
   set_params(params);
 
-  SEXP first_col = params[0];
+  SEXP first_col = cpp11::as_sexp(params[0]);
   groups_ = Rf_length(first_col);
   group_ = 0;
 
@@ -133,42 +147,60 @@ void SqliteResultImpl::bind(const List& params) {
   after_bind(has_params);
 }
 
-List SqliteResultImpl::fetch(const int n_max) {
+cpp11::list SqliteResultImpl::fetch(const int n_max) {
   if (!ready_)
-    stop("Query needs to be bound before fetching");
+    cpp11::stop("Query needs to be bound before fetching");
 
   int n = 0;
-  List out;
+  cpp11::list out;
 
-  if (n_max != 0)
+  if (n_max != 0) {
     out = fetch_rows(n_max, n);
+  }
   else
     out = peek_first_row();
 
   return out;
 }
 
-List SqliteResultImpl::get_column_info() {
+cpp11::list SqliteResultImpl::get_column_info() {
+  using namespace cpp11::literals;
   peek_first_row();
 
-  CharacterVector names(cache.names_.begin(), cache.names_.end());
+  cpp11::writable::strings names(cache.names_.size());
+  auto it = cache.names_.begin();
+  for (int i = 0; i < names.size(); i++, it++)
+    names[i] = *it;
 
-  CharacterVector types(cache.ncols_);
+  cpp11::writable::strings types(cache.ncols_);
   for (size_t i = 0; i < cache.ncols_; i++) {
-    types[i] = Rf_type2char(DbColumnStorage::sexptype_from_datatype(types_[i]));
+    switch(types_[i]) {
+    case DT_DATE:
+      types[i] = "Date";
+      break;
+    case DT_DATETIME:
+      types[i] = "POSIXct";
+      break;
+    case DT_TIME:
+      types[i] = "hms";
+      break;
+    default:
+      types[i] = Rf_type2char(DbColumnStorage::sexptype_from_datatype(types_[i]));
+      break;
+    }
   }
 
-  return List::create(names, types);
+  return cpp11::list({"name"_nm = names, "type"_nm = types});
 }
 
 
 
 // Publics (custom) ////////////////////////////////////////////////////////////
 
-CharacterVector SqliteResultImpl::get_placeholder_names() const {
+cpp11::strings SqliteResultImpl::get_placeholder_names() const {
   int n = sqlite3_bind_parameter_count(stmt);
 
-  CharacterVector res(n);
+  cpp11::writable::strings res(n);
 
   for (int i = 0; i < n; ++i) {
     const char* placeholder_name = sqlite3_bind_parameter_name(stmt, i + 1);
@@ -176,7 +208,7 @@ CharacterVector SqliteResultImpl::get_placeholder_names() const {
       placeholder_name = "";
     else
       ++placeholder_name;
-    res[i] = String(placeholder_name, CE_UTF8);
+    res[i] = placeholder_name;
   }
 
   return res;
@@ -186,7 +218,7 @@ CharacterVector SqliteResultImpl::get_placeholder_names() const {
 
 // Privates ////////////////////////////////////////////////////////////////////
 
-void SqliteResultImpl::set_params(const List& params) {
+void SqliteResultImpl::set_params(const cpp11::list& params) {
   params_ = params;
 }
 
@@ -259,11 +291,11 @@ void SqliteResultImpl::bind_parameter_pos(int j, SEXP value_) {
       sqlite3_bind_blob(stmt, j, RAW(value), Rf_length(value), SQLITE_TRANSIENT);
     }
     else {
-      stop("Can only bind lists of raw vectors (or NULL)");
+      cpp11::stop("Can only bind lists of raw vectors (or NULL)");
     }
   }
   else {
-    stop("Don't know how to handle parameter of type %s.",
+    cpp11::stop("Don't know how to handle parameter of type %s.",
          Rf_type2char(TYPEOF(value_)));
   }
 }
@@ -274,13 +306,13 @@ void SqliteResultImpl::after_bind(bool params_have_rows) {
     step();
 }
 
-List SqliteResultImpl::fetch_rows(const int n_max, int& n) {
+cpp11::list SqliteResultImpl::fetch_rows(const int n_max, int& n) {
   n = (n_max < 0) ? 100 : n_max;
 
-  SqliteDataFrame data(stmt, cache.names_, n_max, types_);
+  SqliteDataFrame data(stmt, cache.names_, n_max, types_, with_alt_types_);
 
   if (complete_ && data.get_ncols() == 0) {
-    warning("Don't need to call dbFetch() for statements, only for queries");
+    Rf_warning("`dbGetQuery()`, `dbSendQuery()` and `dbFetch()` should only be used with `SELECT` queries. Did you mean `dbExecute()`, `dbSendStatement()` or `dbGetRowsAffected()`?");
   }
 
   while (!complete_) {
@@ -329,8 +361,8 @@ bool SqliteResultImpl::step_done() {
   return more_params;
 }
 
-List SqliteResultImpl::peek_first_row() {
-  SqliteDataFrame data(stmt, cache.names_, 1, types_);
+cpp11::list SqliteResultImpl::peek_first_row() {
+  SqliteDataFrame data(stmt, cache.names_, 1, types_, with_alt_types_);
 
   if (!complete_)
     data.set_col_values();
@@ -344,5 +376,5 @@ void SqliteResultImpl::raise_sqlite_exception() const {
 }
 
 void SqliteResultImpl::raise_sqlite_exception(sqlite3* conn) {
-  stop(sqlite3_errmsg(conn));
+  cpp11::stop(sqlite3_errmsg(conn));
 }
